.xavier-mdt-icon-wrapper {
  display: inline-block;
  width: var(--xavier-icon-width);
  height: 100%;
  text-align: center;
  vertical-align: top;
  line-height: calc(100% + 15px); /* Maybe hard code */
  cursor: pointer;
}

.xavier-mdt-icon-wrapper:hover {
  background-color: var(--jp-layout-color2);
}

.xavier-mdt-icon-wrapper.xavier-mdt-icon-placeholder {
  cursor: default;
}

.xavier-mdt-icon-wrapper.xavier-mdt-icon-placeholder:hover {
  background-color: transparent;
}