export const ViewMode = {
  INIT: 0,
  FILTER_COLUMN: 1,
  FILTER_VALUE: 2
};

export const ViewIconStyle = {
  fill: "#777777",
}

// SpreadSheet column Highlight color
export const SSRowHLDelColor = "#f5eef7";

export const SSColumnHLColor = "#e9e9ff" //"#dfdfff" //"#d5d4ff";

export const SSColumnPreColor = "#fffcee";

export const SSRowDelColor = "#fff2f0";

export const SSAlternateColor = ["#f1f1f1", "#ffffff"] // "#eaebee"

export const SSCellMaxWidth = 75;

export const SSTableRowHeight = 30;

export const SSMaxHeight = 8.5 * SSTableRowHeight;