.xavier-ss-wrapper {
  display: block;
  position: relative;
  width: 100%;
  height: calc(100% - 30px);
  overflow: auto;
}

.xavier-dvt-name.xavier-st-text {
  width: 100px;
}

.xavier-cc-float-right {
  position: sticky; /* Let it horizontally fixed but vertically not fixed */
  /* right: 10px; */ /* the right is not fixed */
  opacity: 0.9;
  /* box-shadow: 0px 4px 8px rgba(0, 0, 0, 0.3); */
  background-color: white;
  z-index: 1000;
}